/*
	ZPORTS.C -- Serial Support for Zenith Z-100 (2661 EPCI)

	PC89LINK: HDOS <-> MSDOS File Transfer Package

	Copyright (c) 1989 Lindley Systems
			   4257 Berwick Place
			   Woodbridge, VA 22192


*/

#include <conio.h>
extern int portnum;

/* Prototypes: */

unsigned char nget();
void setup ();				/* set up I/O ports */
int nports();				/* Number of defined ports */
void nput (char c);			/* send	c to port */
int tget();				/* Get a character, with time-out */
int tstport();				/* test	input port */
void initserial();			/* Set up interrupts &c. */
void resetserial();

#define CHARTIME 15000		/* Timeout delay for each character */

#define MAXPORT 2			/* COM2: is last number */
int port = 0;

int maxbaud = 6;
long baudtable[6]={1200,2400,4800,9600,19200,38400};
int baudent = 3;			/* Entry in baudrate table to use */

static int uart_baud[6]={8,10,12,13,14,15};

static int pic_imr;			/* 8259 Progrmbl Int Ctrl Int Mask Reg */
static int epci_stat;			/* Current serial chip status */
static int epci_setup;			/* What to output to clear errors */

#define EPCI_NORM 0x15
#define EPCI_LOOP 0xB7

void setup ()				/* set up I/O ports */
{ unsigned int baud;
  int baudindex;
  int intenab;

  port = 0xE8 + portnum*4;		/* E8 for J1, EC for J2 */
  outp (0xF3,pic_imr | 0x30);		/* Disable Serial A & B interrupts */
  inp(0xF3);				/* Read back to reset chip's counter */

  baudindex = uart_baud[baudent];
  epci_setup = EPCI_NORM;

  outp(port + 2, 0x4E);			/* 1 Stop, No Parity, 8 bits, Asynch */
  outp(port + 2, 0xF0 + baudindex);	/* Enable clocks, set baud rate */
  outp(port + 3, epci_setup);		/* Normal op, set RTS hi, reset errors */
					/* and enable Rcvr and Xmitter */
}

void initserial()
{
  pic_imr = inp(0xF3);			/* Currently enabled interrupts */
}

void resetserial()
{
  outp (0xF3,pic_imr);			/* Go back to original interrupt mask */
  inp(0xF3);				/* Read back to reset counter */
}

void loopback(int loop_on)
{
  epci_setup = loop_on?EPCI_LOOP:EPCI_NORM;
  outp(port + 3, epci_setup);
}

int nports()
{
  return(2);				/* Always 2 ports */
}

void nput (char c)			/* send	c to port */
{
  while	((inp(port + 1)	& 1) == 0);	/* wait	till transmitter ready */
  outp(port, c);			/* send	the char */
/* outp(port+3,epci_setup); */
}

/* TGET - Timeout Get character. Return character, or -1 if none */

int tget()				/* Get a character, with time-out */
{ int t=1000;				/* Countdown delay */
	while (tstport() <= 0)
	  if (--t ==0)
	    return(-1);			/* No character */
	return((unsigned)inp(port));	/* Return char */
}

unsigned char nget ()			/* get a character */
{
  while	(tstport () <= 0);		/* wait	for a character	*/
  return (unsigned)inp(port);		/* read	char, and return it */
}

int tstport ()				/* test	input port */
{ epci_stat = inp(port+1);
  if (epci_stat & 0x30)			/* Framing or overrun error? */
    outp(port+3,epci_setup);		/* Reset error */
  return ((epci_stat & 2)==0?0:(epci_stat & 0x30)?-1:1);
					/* 0=no data, 1=data, -1=error */
}
